/*
Example for UT_7_SEG Click

    Date          : Jun 2017.
    Author        : Danilo M.

Test configuration dsPIC :
    
    MCU             : P33FJ256GP710A
    Dev. Board      : EasyPIC Fusion v7
    SW              : mikroC PRO for dsPIC v7.0.1

Description :

    This example shows countdown from 99 to 00, the LED driver Max6969 is programmed via SPI,
    and the brightness of 7 seg display is regulated by duty cycle of PWM.

*/

#include "stdint.h"

sbit   MAX6969_LE_PIN   at  RC2_bit;
sbit   MAX6969_LE_PIN_Direction   at  TRISC2_bit;

uint16_t pwmPeriod;
static const uint8_t minus_char = 0x40;

void systemInit()
{
    CLKDIVbits.PLLPRE = 0;   // PLLPRE<4:0> = 0  ->  N1 = 2    8MHz / 2 = 4MHz
                             // (must be within 0.8 MHz to 8 MHz range)
    PLLFBD =   38;           // PLLDIV<8:0> = 38 ->  M = 40    4MHz * 40 = 160MHz
                             // (must be within 100 MHz to 200 MHz range)
    CLKDIVbits.PLLPOST = 0;  // PLLPOST<1:0> = 0 ->  N2 = 2    160MHz / 2 = 80MHz
                             // (must be within 12.5 MHz to 80 MHz range)

    AD1PCFGL = 0xFFFE;       // Set All pins as digital
    AD1PCFGH = 0xFFFF;
    MAX6969_LE_PIN_Direction = 0;
    SPI1_Init();
    pwmPeriod = PWM_Init(5000, 1, 1, 2);
    PWM_Start(1);
    PWM_Set_Duty(pwmPeriod/10, 1);

}

void MAX6969_Chip_Select()
{
    MAX6969_LE_PIN = 1;
    asm nop
    asm nop
    asm nop
    MAX6969_LE_PIN = 0;
}

static void U7SEG_Write(uint8_t number)
{
    char numbers[10] = {0x3F, 0x06, 0x5B, 0x4F, 0x66, 0x6D, 0x7D, 0x07, 0x7F, 0x6F};
    uint8_t tens = number / 10;
    uint8_t ones = number % 10;
    if (number > 99) return;

    SPI_Wr_Ptr(numbers[ones]);
    SPI_Wr_Ptr(numbers[tens]);
}

void main()
{
    uint8_t counter;

    systemInit();
    counter = 100;

    while(counter--)
    {
        U7SEG_Write(counter);
        MAX6969_Chip_Select();
        Delay_ms(500);
    }

    SPI_Wr_Ptr(minus_char);
    SPI_Wr_Ptr(minus_char);
    MAX6969_Chip_Select();
}